@@ -17,12 +17,14 @@
 
 from contextlib import contextmanager, closing
 import hashlib
+import json
 import logging
 import os
 from uuid import uuid4
 import sys
 import time
 import errno
+import six
 import six.moves.cPickle as pickle
 from swift import gettext_ as _
 from tempfile import mkstemp
@@ -30,8 +32,9 @@ from tempfile import mkstemp
 from eventlet import sleep, Timeout
 import sqlite3
 
-from swift.common.constraints import MAX_META_COUNT, MAX_META_OVERALL_SIZE
-from swift.common.utils import json, Timestamp, renamer, \
+from swift.common.constraints import MAX_META_COUNT, MAX_META_OVERALL_SIZE, \
+    check_utf8
+from swift.common.utils import Timestamp, renamer, \
     mkdirs, lock_parent_directory, fallocate
 from swift.common.exceptions import LockTimeout
 from swift.common.swob import HTTPBadRequest
@@ -43,16 +46,18 @@ DB_PREALLOCATION = False
 BROKER_TIMEOUT = 25
 #: Pickle protocol to use
 PICKLE_PROTOCOL = 2
-#: Max number of pending entries
+#: Max size of .pending file in bytes. When this is exceeded, the pending
+# records will be merged.
 PENDING_CAP = 131072
 
 
 def utf8encode(*args):
-    return [(s.encode('utf8') if isinstance(s, unicode) else s) for s in args]
+    return [(s.encode('utf8') if isinstance(s, six.text_type) else s)
+            for s in args]
 
 
 def utf8encodekeys(metadata):
-    uni_keys = [k for k in metadata if isinstance(k, unicode)]
+    uni_keys = [k for k in metadata if isinstance(k, six.text_type)]
     for k in uni_keys:
         sv = metadata[k]
         del metadata[k]
@@ -331,7 +336,7 @@ class DatabaseBroker(object):
         elif 'disk I/O error' in str(exc_value):
             exc_hint = 'disk error while accessing'
         else:
-            raise exc_type, exc_value, exc_traceback
+            six.reraise(exc_type, exc_value, exc_traceback)
         prefix_path = os.path.dirname(self.db_dir)
         partition_path = os.path.dirname(prefix_path)
         dbs_path = os.path.dirname(partition_path)
@@ -346,8 +351,10 @@ class DatabaseBroker(object):
                 raise
             quar_path = "%s-%s" % (quar_path, uuid4().hex)
             renamer(self.db_dir, quar_path, fsync=False)
-        detail = _('Quarantined %s to %s due to %s database') % \
-                  (self.db_dir, quar_path, exc_hint)
+        detail = _('Quarantined %(db_dir)s to %(quar_path)s due to '
+                   '%(exc_hint)s database') % {'db_dir': self.db_dir,
+                                               'quar_path': quar_path,
+                                               'exc_hint': exc_hint}
         self.logger.error(detail)
         raise sqlite3.DatabaseError(detail)
 
@@ -625,7 +632,7 @@ class DatabaseBroker(object):
             with lock_parent_directory(self.pending_file,
                                        self.pending_timeout):
                 self._commit_puts()
-        except LockTimeout:
+        except (LockTimeout, sqlite3.OperationalError):
             if not self.stale_reads_ok:
                 raise
 
@@ -726,11 +733,11 @@ class DatabaseBroker(object):
     @staticmethod
     def validate_metadata(metadata):
         """
-        Validates that metadata_falls within acceptable limits.
+        Validates that metadata falls within acceptable limits.
 
         :param metadata: to be validated
         :raises: HTTPBadRequest if MAX_META_COUNT or MAX_META_OVERALL_SIZE
-                 is exceeded
+                 is exceeded, or if metadata contains non-UTF-8 data
         """
         meta_count = 0
         meta_size = 0
@@ -744,6 +751,10 @@ class DatabaseBroker(object):
                 key = key[len(prefix):]
                 meta_count = meta_count + 1
                 meta_size = meta_size + len(key) + len(value)
+            bad_key = key and not check_utf8(key)
+            bad_value = value and not check_utf8(value)
+            if bad_key or bad_value:
+                raise HTTPBadRequest('Metadata must be valid UTF-8')
         if meta_count > MAX_META_COUNT:
             raise HTTPBadRequest('Too many metadata items; max %d'
                                  % MAX_META_COUNT)
