@@ -132,7 +132,7 @@ backwards compatibility. At first chance, set ``object_post_as_copy`` to
 """
 
 import os
-from ConfigParser import ConfigParser, NoSectionError, NoOptionError
+from six.moves.configparser import ConfigParser, NoSectionError, NoOptionError
 from six.moves.urllib.parse import quote, unquote
 
 from swift.common import utils
@@ -455,10 +455,33 @@ class ServerSideCopyMiddleware(object):
             close_if_possible(source_resp.app_iter)
             return source_resp(source_resp.environ, start_response)
 
-        # Create a new Request object based on the original req instance.
-        # This will preserve env and headers.
-        sink_req = Request.blank(req.path_info,
-                                 environ=req.environ, headers=req.headers)
+        # Create a new Request object based on the original request instance.
+        # This will preserve original request environ including headers.
+        sink_req = Request.blank(req.path_info, environ=req.environ)
+
+        def is_object_sysmeta(k):
+            return is_sys_meta('object', k)
+
+        if 'swift.post_as_copy' in sink_req.environ:
+            # Post-as-copy: ignore new sysmeta, copy existing sysmeta
+            remove_items(sink_req.headers, is_object_sysmeta)
+            copy_header_subset(source_resp, sink_req, is_object_sysmeta)
+        elif config_true_value(req.headers.get('x-fresh-metadata', 'false')):
+            # x-fresh-metadata only applies to copy, not post-as-copy: ignore
+            # existing user metadata, update existing sysmeta with new
+            copy_header_subset(source_resp, sink_req, is_object_sysmeta)
+            copy_header_subset(req, sink_req, is_object_sysmeta)
+        else:
+            # First copy existing sysmeta, user meta and other headers from the
+            # source to the sink, apart from headers that are conditionally
+            # copied below and timestamps.
+            exclude_headers = ('x-static-large-object', 'x-object-manifest',
+                               'etag', 'content-type', 'x-timestamp',
+                               'x-backend-timestamp')
+            copy_header_subset(source_resp, sink_req,
+                               lambda k: k.lower() not in exclude_headers)
+            # now update with original req headers
+            sink_req.headers.update(req.headers)
 
         params = sink_req.params
         if params.get('multipart-manifest') == 'get':
@@ -466,8 +489,9 @@ class ServerSideCopyMiddleware(object):
                 params['multipart-manifest'] = 'put'
             if 'X-Object-Manifest' in source_resp.headers:
                 del params['multipart-manifest']
-                sink_req.headers['X-Object-Manifest'] = \
-                    source_resp.headers['X-Object-Manifest']
+                if 'swift.post_as_copy' not in sink_req.environ:
+                    sink_req.headers['X-Object-Manifest'] = \
+                        source_resp.headers['X-Object-Manifest']
             sink_req.params = params
 
         # Set data source, content length and etag for the PUT request
@@ -489,32 +513,19 @@ class ServerSideCopyMiddleware(object):
         else:
             # since we're not copying the source etag, make sure that any
             # container update override values are not copied.
-            remove_items(source_resp.headers, lambda k: k.startswith(
+            remove_items(sink_req.headers, lambda k: k.startswith(
                 'X-Object-Sysmeta-Container-Update-Override-'))
 
         # We no longer need these headers
         sink_req.headers.pop('X-Copy-From', None)
         sink_req.headers.pop('X-Copy-From-Account', None)
+
         # If the copy request does not explicitly override content-type,
         # use the one present in the source object.
         if not req.headers.get('content-type'):
             sink_req.headers['Content-Type'] = \
                 source_resp.headers['Content-Type']
 
-        fresh_meta_flag = config_true_value(
-            sink_req.headers.get('x-fresh-metadata', 'false'))
-
-        if fresh_meta_flag or 'swift.post_as_copy' in sink_req.environ:
-            # Post-as-copy: ignore new sysmeta, copy existing sysmeta
-            condition = lambda k: is_sys_meta('object', k)
-            remove_items(sink_req.headers, condition)
-            copy_header_subset(source_resp, sink_req, condition)
-        else:
-            # Copy/update existing sysmeta, transient-sysmeta and user meta
-            _copy_headers(source_resp.headers, sink_req.headers)
-            # Copy/update new metadata provided in request if any
-            _copy_headers(req.headers, sink_req.headers)
-
         # Create response headers for PUT response
         resp_headers = self._create_response_headers(source_path,
                                                      source_resp, sink_req)
